@@ -135,8 +135,8 @@ screen_cast_session_new (GVariant *options,
                             "impl-dbus-name", impl_dbus_name,
                             NULL);
 
-  g_debug ("screen cast session owned by '%s' created",
-           session->sender);
+  if (session)
+    g_debug ("screen cast session owned by '%s' created", session->sender);
 
   return (ScreenCastSession*)session;
 }
