@@ -71,8 +71,7 @@ send_response_in_thread_func (GTask *task,
   guint response;
   GVariant *options;
   g_autoptr(GError) error = NULL;
-  const char *uri;
-  g_autofree char *ruri = NULL;
+  const char *retval;
 
   REQUEST_AUTOLOCK (request);
 
@@ -84,17 +83,40 @@ send_response_in_thread_func (GTask *task,
   if (response != 0)
     goto out;
 
-  if (!g_variant_lookup (options, "uri", "&s", &uri))
+  retval = g_object_get_data (G_OBJECT (task), "retval");
+  if (g_strcmp0 (retval, "url") == 0)
     {
-      g_warning ("No URI was provided");
-      goto out;
-    }
+      const char *uri;
+      g_autofree char *ruri = NULL;
 
-  ruri = register_document (uri, xdp_app_info_get_id (request->app_info), FALSE, FALSE, &error);
-  if (ruri == NULL)
-    g_warning ("Failed to register %s: %s", uri, error->message);
+      if (!g_variant_lookup (options, "uri", "&s", &uri))
+        {
+          g_warning ("No URI was provided");
+          goto out;
+        }
+
+      ruri = register_document (uri, xdp_app_info_get_id (request->app_info), FALSE, FALSE, &error);
+      if (ruri == NULL)
+        g_warning ("Failed to register %s: %s", uri, error->message);
+      else
+        g_variant_builder_add (&results, "{&sv}", "uri", g_variant_new_string (ruri));
+    }
+  else if (g_strcmp0 (retval, "color") == 0)
+    {
+      double red, green, blue;
+
+      if (!g_variant_lookup (options, "color", "(ddd)", &red, &green, &blue))
+        {
+          g_warning ("No color was provided");
+          goto out;
+        }
+
+      g_variant_builder_add (&results, "{&sv}", "color", g_variant_new ("(ddd)", red, green, blue));
+    }
   else
-    g_variant_builder_add (&results, "{&sv}", "uri", g_variant_new_string (ruri));
+    {
+      g_warning ("Don't know what to return");
+    }
 
 out:
   if (request->exported)
@@ -132,6 +154,7 @@ screenshot_done (GObject *source,
 
   task = g_task_new (NULL, NULL, NULL, NULL);
   g_task_set_task_data (task, g_object_ref (request), g_object_unref);
+  g_object_set_data (G_OBJECT (task), "retval", "url");
   g_task_run_in_thread (task, send_response_in_thread_func);
 }
 
@@ -185,16 +208,95 @@ handle_screenshot (XdpScreenshot *object,
   return TRUE;
 }
 
+static void
+pick_color_done (GObject *source,
+                 GAsyncResult *result,
+                 gpointer data)
+{
+  g_autoptr(Request) request = data;
+  guint response = 2;
+  g_autoptr(GVariant) options = NULL;
+  g_autoptr(GError) error = NULL;
+  g_autoptr(GTask) task = NULL;
+
+  if (!xdp_impl_screenshot_call_pick_color_finish (XDP_IMPL_SCREENSHOT (source),
+                                                   &response,
+                                                   &options,
+                                                   result,
+                                                   &error))
+    {
+      g_warning ("A backend call failed: %s", error->message);
+    }
+
+  g_object_set_data (G_OBJECT (request), "response", GINT_TO_POINTER (response));
+  if (options)
+    g_object_set_data_full (G_OBJECT (request), "options", g_variant_ref (options), (GDestroyNotify)g_variant_unref);
+
+  task = g_task_new (NULL, NULL, NULL, NULL);
+  g_task_set_task_data (task, g_object_ref (request), g_object_unref);
+  g_object_set_data (G_OBJECT (task), "retval", "color");
+  g_task_run_in_thread (task, send_response_in_thread_func);
+}
+
+static XdpOptionKey pick_color_options[] = {
+};
+
+static gboolean
+handle_pick_color (XdpScreenshot *object,
+                   GDBusMethodInvocation *invocation,
+                   const gchar *arg_parent_window,
+                   GVariant *arg_options)
+{
+  Request *request = request_from_invocation (invocation);
+  g_autoptr(GError) error = NULL;
+  g_autoptr(XdpImplRequest) impl_request = NULL;
+  GVariantBuilder opt_builder;
+
+  REQUEST_AUTOLOCK (request);
+
+  impl_request = xdp_impl_request_proxy_new_sync (g_dbus_proxy_get_connection (G_DBUS_PROXY (impl)),
+                                                  G_DBUS_PROXY_FLAGS_NONE,
+                                                  g_dbus_proxy_get_name (G_DBUS_PROXY (impl)),
+                                                  request->id,
+                                                  NULL, &error);
+  if (!impl_request)
+    {
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      return TRUE;
+    }
+
+  request_set_impl_request (request, impl_request);
+  request_export (request, g_dbus_method_invocation_get_connection (invocation));
+
+  g_variant_builder_init (&opt_builder, G_VARIANT_TYPE_VARDICT);
+  xdp_filter_options (arg_options, &opt_builder,
+                      pick_color_options, G_N_ELEMENTS (pick_color_options));
+
+  xdp_impl_screenshot_call_pick_color (impl,
+                                       request->id,
+                                       xdp_app_info_get_id (request->app_info),
+                                       arg_parent_window,
+                                       g_variant_builder_end (&opt_builder),
+                                       NULL,
+                                       pick_color_done,
+                                       g_object_ref (request));
+
+  xdp_screenshot_complete_pick_color (object, invocation, request->id);
+
+  return TRUE;
+}
+
 static void
 screenshot_iface_init (XdpScreenshotIface *iface)
 {
   iface->handle_screenshot = handle_screenshot;
+  iface->handle_pick_color = handle_pick_color;
 }
 
 static void
 screenshot_init (Screenshot *fc)
 {
-  xdp_screenshot_set_version (XDP_SCREENSHOT (fc), 1);
+  xdp_screenshot_set_version (XDP_SCREENSHOT (fc), 2);
 }
 
 static void
